﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Delta_Test
{
	/// <summary>
	/// Вспомогательный класс для заполнения прочитанных данных от параметров на форме программы
	/// </summary>
	public class DeltaParameterOnApplicationForm
	{
		/// <summary>
		/// Метка на форме, где будет помещено прочитанное значение параметра
		/// </summary>
		private Label label;
		/// <summary>
		/// Параметр, значение которого отображается на форме
		/// </summary>
		private DeltaModBusASCII.Parameter parameter;
		/// <summary>
		/// Соотв. делитель для прочитанной целой величины параметра, чтобы преобразовать ее в соотв. дробную величину
		/// </summary>
		private int dividerForParameter;

		/// <summary>
		/// Конструктор вспомогательного класса для заполнеия формы данных от прочитанных параметров преобразователя
		/// </summary>
		/// <param name="l"> Метка на форме, куда пишется строковое значение параметра </param>
		/// <param name="p"> Параметр, значение которого отображается на форме </param>
		/// <param name="divider"> Соотв. делитель для прочитанной целой величины параметра, чтобы преобразовать ее в соотв. дробную величину </param>
		public DeltaParameterOnApplicationForm(Label l, DeltaModBusASCII.Parameter p, int divider)
		{
			label = l;
			parameter = p;
			dividerForParameter = divider;
		}

		/// <summary>
		/// Заполняет свою текстовую метку соотв. прочитанным значением параметра
		/// </summary>
		/// <param name="modbusObject"> Объект класса для работы с преобразователями Delta серии CP2000 по протоколу MODBUS ASCII. </param>
		public bool FillCurrentLabel(DeltaModBusASCII modbusObject)
		{
			ushort parametrValue;

			if (modbusObject.ReadOneParameterValue(parameter, out parametrValue) == true)
			{
				// если соотв. делитель равен 1, то число параметра показываем как целое
				if (dividerForParameter == 1)
				{
					short signedValue = (short)parametrValue;
					label.Text = signedValue.ToString();
				}
				// иначе - как дробное и с нужным кол-вом знаков после дес. точки
				else
				{
					label.Text = ((float)parametrValue / dividerForParameter).ToString("N" + (dividerForParameter.ToString().Length - 1).ToString());
				}
	
				return true;
			}
			else
			{
				return false;
			}
		}
	}
}
