﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace Delta_Test
{
	public partial class Form1 : Form
	{
        /// <summary>
		/// Объект класса для работы с преобразователями Delta серии CP2000 по протоколу MODBUS ASCII.
        /// </summary>
		private DeltaModBusASCII DeltaModbus;

		public Form1()
		{
			InitializeComponent();
			string[] portnames = SerialPort.GetPortNames();
			PortNameComboBox.Items.Clear();
			PortNameComboBox.Items.AddRange(portnames);
			if (PortNameComboBox.Items.Count > 0)
			{
				PortNameComboBox.SelectedIndex = 0;
				// по умолчанию - 9600
				BaudRateComboBox.Enabled = true;
				BaudRateComboBox.SelectedIndex = 2;
				// по умолчанию - None
				ParityComboBox.Enabled = true;
				ParityComboBox.SelectedIndex = 2;
				// по умолчанию - 7
				DataBitsComboBox.Enabled = true;
				DataBitsComboBox.SelectedIndex = 3;
				// по умолчанию - 2
				StopBitsComboBox.Enabled = true;
				StopBitsComboBox.SelectedIndex = 1;
				// кнопки подключения/отключения
				ConnectButton.Enabled = true;
				CloseButton.Enabled = true;
			}
		}

		private void Form1_FormClosed(object sender, FormClosedEventArgs e)
		{
			if (DeltaModbus != null && DeltaModbus.Port.IsOpen)
			{
				DeltaModbus.Port.Close();
			}
		}

		private void ConnectButton_Click(object sender, EventArgs e)
		{
			if (PortNameComboBox.Items.Count > 0 && PortNameComboBox.SelectedIndex >= 0)
			{
				try
				{
					int baudRate, dataBits;

					// считываем с формы настройки компорта (т.к. значения в соотв. ComboBox-ах могут быть изменены юзером, то предварительно их все проверяем)
					string portName = PortNameComboBox.SelectedItem.ToString();
					if (Int32.TryParse(BaudRateComboBox.SelectedItem.ToString(), out baudRate) == false)
					{
						baudRate = 9600;
					}
					if (Int32.TryParse(DataBitsComboBox.SelectedItem.ToString(), out dataBits) == false)
					{
						dataBits = 7;
					}
					string parityStr = ParityComboBox.SelectedItem.ToString();
					Parity parity = (Enum.IsDefined(typeof(Parity), parityStr)) ? (Parity)Enum.Parse(typeof(Parity), parityStr) : Parity.None;
					StopBits stopBits = (StopBits)(StopBitsComboBox.SelectedIndex + 1);
					// Открываем соединение MODBUS ASCII с преобразователем Delta
					DeltaModbus = new DeltaModBusASCII(portName, baudRate, parity, dataBits, stopBits);
					PanelGroupBox.Enabled = true;
					timer1.Start();
				}
				catch(ApplicationException ex)
				{
					MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Stop);
				}
			}
		}

		private void Buttons_Click(object sender, EventArgs e)
		{
			// временно отключаем Таймер1 на форме
			timer1.Enabled = false;

			Button pressedButton = sender as Button;
			if (pressedButton == CloseButton)
			{
				DeltaModbus.Port.Close();
				PanelGroupBox.Enabled = false;
				timer1.Stop();
			}
			else if (pressedButton == RunButton)
			{
				DeltaModbus.SendMessage(DeltaModBusASCII.Function.WRITE_REGISTER, DeltaModBusASCII.Parameter.p2000_COMMAND, (int)DeltaModBusASCII.COMMAND.Bits0_3_RUN);
			}
			else if (pressedButton == StopButton)
			{
				DeltaModbus.SendMessage(DeltaModBusASCII.Function.WRITE_REGISTER, DeltaModBusASCII.Parameter.p2000_COMMAND, (int)DeltaModBusASCII.COMMAND.Bits0_3_STOP);
			}
			else if (pressedButton == FwdRevButton)
			{
				DeltaModbus.SendMessage(DeltaModBusASCII.Function.WRITE_REGISTER, DeltaModBusASCII.Parameter.p2000_COMMAND, (int)DeltaModBusASCII.COMMAND.Bits4_5_CHANGE_DIRECTION);
			}
			else if (pressedButton == TestButton)
			{
				//DeltaModbus.SendMessage(DeltaModBusASCII.Function.WRITE_REGISTER, DeltaModBusASCII.Parameter.p2000_COMMAND, (int)DeltaModBusASCII.COMMAND.Bits13_14__CHANGE_OPERATION_SOURCE);
				//DeltaModbus.SendMessage(DeltaModBusASCII.Function.READ_REGISTERS, DeltaModBusASCII.Parameter.p0028_SwitchingFromAutoHandMode, 1);
				DeltaModbus.SendMessage(DeltaModBusASCII.Function.WRITE_REGISTER, DeltaModBusASCII.Parameter.p0028_SwitchingFromAutoHandMode, 0);
			}
			// делаем паузу для беспроблемной отправки запроса (при 9600 бит/сек отправка и прием ~ по 50 мс каждые)
			Thread.Sleep(50);

			// включаем обратно Таймер1
			timer1.Enabled = true;
		}

		private void timer1_Tick(object sender, EventArgs e)
		{
			if (!DeltaModbus.Port.IsOpen)
			{
				if (DriveGroupBox.Enabled != false)
				{
					DriveGroupBox.Enabled = false;
				}
				return;
			}
			else
			{
				DriveGroupBox.Enabled = true;
			}

			// если уже есть принятая строка ответ (от пред. команды и пр.), то считываем/показываем ее
			if (DeltaModbus.IsStringReceived == true)
			{
				ReceivedDataListBox.Items.Add(DeltaModbus.ReceivedString);
			}

			// теперь отправляем запросы на считывание текущего состояния и параметров преобразователя и двигателя
			ushort parametrValue;
			// сначала отправляем запрос на считывание статуса преобразователя (адрес параметра 21-01)
			if (DeltaModbus.ReadOneParameterValue(DeltaModBusASCII.Parameter.p2101_DRIVESTATUS, out parametrValue) != true)
			{
				//MessageBox.Show("Нет правильного ответа от преобразователя!", "Ошибка!", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			// получаем текущее состояние преобразователя в флагах из ранее считанного параметра 21-01
			DeltaModBusASCII.STATUS status = DeltaModbus.GetDriverStatusFromParametrData(parametrValue);
			// в соответствии с полученными флагами этого состояния усстанавливаем соотв. элементы на форме
			if ((status & DeltaModBusASCII.STATUS.Stops) == DeltaModBusASCII.STATUS.Stops)
			{
				DriveStatusLabel.Text = "остановлен";
			}
			else if ((status & DeltaModBusASCII.STATUS.Standby) == DeltaModBusASCII.STATUS.Standby)
			{
				DriveStatusLabel.Text = "в режиме \"Standby\"";
			}
			else if ((status & DeltaModBusASCII.STATUS.Decelerating) == DeltaModBusASCII.STATUS.Decelerating)
			{
				DriveStatusLabel.Text = "останавливается";
			}
			else if ((status & DeltaModBusASCII.STATUS.Operating) == DeltaModBusASCII.STATUS.Operating)
			{
				DriveStatusLabel.Text = "работает";
			}
			// управление командами преобразователю
			DriveOperationLabel.Text = "команды задаются " + (((status & DeltaModBusASCII.STATUS.OperContrByCommInter) == DeltaModBusASCII.STATUS.OperContrByCommInter) ? "с пульта" : "через RS-485");
			// направление вращения двигателя
			if ((status & DeltaModBusASCII.STATUS.FWDrun) == DeltaModBusASCII.STATUS.FWDrun)
			{
				DirectionLabel.Text = "прямое";
			}
			else if ((status & DeltaModBusASCII.STATUS.FromFWDtoREVrun) == DeltaModBusASCII.STATUS.FromFWDtoREVrun)
			{
				DirectionLabel.Text = "с прямого на обратное";
			}

			else if ((status & DeltaModBusASCII.STATUS.FromREVtoFWDrun) == DeltaModBusASCII.STATUS.FromREVtoFWDrun)
			{
				DirectionLabel.Text = "с обратного на прямое";
			}

			else if ((status & DeltaModBusASCII.STATUS.REVrun) == DeltaModBusASCII.STATUS.REVrun)
			{
				DirectionLabel.Text = "обратное";
			}

			FillLabelsOnFormWithDriveAndMotorProperties();
		}

		private void FillLabelsOnFormWithDriveAndMotorProperties()
		{
			// массивы с соотв. Label-ами на форме, а также соотв. им параметры и делители (1, 10, 100 или 1000) прочитанных числовых величин этих параметров
			List<DeltaParameterOnApplicationForm> parametersOnForm = new List<DeltaParameterOnApplicationForm>
			{
				new DeltaParameterOnApplicationForm(ActualSpeedLabel, DeltaModBusASCII.Parameter.p210C_ACTUALMOTORSPEED, 1),
				new DeltaParameterOnApplicationForm(TemperatureIGBTLabel, DeltaModBusASCII.Parameter.p220E_TEMPERATURE_IGBT, 10),
				new DeltaParameterOnApplicationForm(TemperatureLabel, DeltaModBusASCII.Parameter.p220F_TEMPERATURE_INSIDE, 10),
				new DeltaParameterOnApplicationForm(TorqueLabel, DeltaModBusASCII.Parameter.p210B_OUTPUT_TORQUE, 1),
				new DeltaParameterOnApplicationForm(OutputVoltageLabel, DeltaModBusASCII.Parameter.p2106_OUTPUT_VOLTAGE, 10),
				new DeltaParameterOnApplicationForm(CounterOfDaysLabel, DeltaModBusASCII.Parameter.p0532_MotorOperationTime_Days, 1),
				new DeltaParameterOnApplicationForm(CounterOfMinutesLabel, DeltaModBusASCII.Parameter.p0531_MotorOperationTime_Min, 1),
				new DeltaParameterOnApplicationForm(OutputCurrentLabel, DeltaModBusASCII.Parameter.p2104_OUTPUT_CURRENT, 100),
				new DeltaParameterOnApplicationForm(PowerFactorLabel, DeltaModBusASCII.Parameter.p210A_POWER_FACTOR_ANGLE, 10),
				new DeltaParameterOnApplicationForm(OutputFrequencyLabel, DeltaModBusASCII.Parameter.p2103_OUTPUT_FREQUENCY, 100),
				new DeltaParameterOnApplicationForm(OutputPowerLabel, DeltaModBusASCII.Parameter.p210F_POWER_OUTPUT, 1000),
				new DeltaParameterOnApplicationForm(OutputCurrentDisplayLabel, DeltaModBusASCII.Parameter.p2200_DISPLAY_OUTPUT_CURRENT, 100)
			};

			// в цикле заполняем все соотв. метки на форме
			foreach (var par in parametersOnForm)
			{
				par.FillCurrentLabel(DeltaModbus);
			}
		}
	}
}
